"""
Словарь неправильно лемматизируемых слов.

Например, вместо леммы 'основной'
выдается лемма 'основный'.

Также можно преобразовать одни леммы на другие.
Например, 'частенько' -> 'часто'.
"""


fixed_lemmas = {
    "арен": "арена",
    "бело-голубым": "бело-голубой",
    "блокшота": "блокшот",
    "большинстве": "большинство",
    "бортик": "борт",
    "буллитная": "буллитный",
    "буллиты": "буллит",
    "бумажка": "бумага",
    "бывает": "бывать",
    "бывало": "бывать",
    "бьемся": "биться",
    "везет": "везти",
    "вели": "везти",
    "вести": "везти",
    "во": "в",
    "вообще-то": "вообще",
    "впритирку": "рядом",
    "временной": "временный",
    "вспомним": "вспомнить",
    "вспомните": "вспомнить",
    "выжали": "выжать",
    "говорили": "говорить",
    "говоря": "говорить",
    "года": "год",
    "голов": "гол",
    "голова": "гол",
    "голы": "гол",
    "дай": "дать",
    "да-да": "да",
    "девчонка": "девочка",
    "деньга": "деньги",
    "длиннее": "длинный",
    "добивании": "добивание",
    "добиваний": "добивание",
    "добивания": "добивание",
    "должок": "долг",
    "домой": "дом",
    "думается": "думаться",
    "думали": "думать",
    "думаю": "думать",
    "жалко": "жаль",
    "желаем": "желать",
    "забиваем": "забивать",
    "запасный": "запасной",
    "забей": "забить",
    "завести": "завезти",
    "защ": "защитник",
    "звездочка": "звезда",
    "звоночек": "звонок",
    "зевнули": "зевнуть",
    "злее": "злой",
    "знали": "знать",
    "значит": "значить",
    "знаю": "знать",
    "зрителя": "зритель",
    "играем": "играть",
    "играй": "играть",
    "играл": "играть",
    "играли": "играть",
    "игралось": "играться",
    "исполнилось": "исполниться",
    "используйте": "использовать",
    "казалось": "казаться",
    "каков": "какой",
    "касается": "касаться",
    "книжка": "книга",
    "которого": "который",
    "красивее": "красивый",
    "крути": "крутить",
    "кулачок": "кулак",
    "лавочка": "лавка",
    "ленточка": "лента",
    "локоток": "локоть",
    "лошадка": "лошадь",
    "маленько": "немного",
    "мальчишка": "мальчик",
    "машинка": "машина",
    "минское": "минский",
    "минутах": "минута",
    "минутка": "минута",
    "минуть": "миновать",
    "млн": "миллион",
    "многих": "многие",
    "многому": "многое",
    "молодая": "молодой",
    "молодчик": "молодец",
    "моментик": "момент",
    "мост": "мостик",
    "мужик": "мужчина",
    "наверное": "наверно",
    "напомним": "напомнить",
    "нет-нет": "нет",
    "ничей": "ничья",
    "нужнее": "нужно",
    "нынче": "сейчас",
    "нюансов": "нюанс",
    "об": "о",
    "обратите": "обратить",
    "обращаем": "обращать",
    "общаетесь": "общаться",
    "овертайме": "овертайм",
    "один-единственный": "единственный",
    "ожидали": "ожидать",
    "основный": "основной",
    "отвести": "отвезти",
    "отдохнули": "отдохнуть",
    "отмечу": "отметить",
    "отрезками": "отрезок",
    "очень-очень": "очень",
    "очень-то": "очень",
    "очки": "очко",
    "ошибаюсь": "ошибаться",
    "плюсик": "плюс",
    "повезло": "повезти",
    "повести": "повезти",
    "повторимся": "повториться",
    "повторюсь": "повториться",
    "подведите": "подвести",
    "получилось": "получиться",
    "получше": "лучше",
    "помните": "помнить",
    "понимали": "понимать",
    "посмотрите": "посмотреть",
    "постарше": "старше",
    "потихонечку": "потихоньку",
    "предыдущий": "прошлый",
    "придется": "прийтись",
    "программка": "программа",
    "пройдет": "пройти",
    "проспали": "проспать",
    "прочтете": "прочитать",
    "прямик": "прямиком",
    "пускай": "пусть",
    "рада": "рад",
    "радует": "радовать",
    "разбираем": "разбирать",
    "размесятся": "размеситься",
    "разу": "раз",
    "рановато": "рано",
    "ребятам": "ребята",
    "ребятами": "ребята",
    "регулярок": "регулярка",
    "рижское": "рижский",
    "родный": "родной",
    "руб": "рубль",
    "рыбка": "нырок",
    "сами": "сам",
    "сейвами": "сейв",
    "сейвище": "сейв",
    "сейвова": "сейв",
    "сейвовый": "сейв",
    "секунд": "секунда",
    "силенка": "сила",
    "скажем": "сказать",
    "сложновато": "сложно",
    "смотрите": "смотреть",
    "со": "с",
    "сопернику": "соперник",
    "спрашивайте": "спрашивать",
    "среднее": "средний",
    "стараешься": "стараться",
    "стенка": "стена",
    "страничка": "страница",
    "ступенька": "ступень",
    "суметь": "смочь",
    "считаете": "считать",
    "сыграем": "сыграть",
    "сыграли": "сыграть",
    "сэйвах": "сейв",
    "сэйвов": "сейв",
    "сэйвова": "сейв",
    "таков": "такой",
    "типа": "тип",
    "того": "тот",
    "только-только": "только",
    "тоненький": "тонкий",
    "тотала": "тотал",
    "траффик": "трафик",
    "тут-то": "тут",
    "тыс": "тысяча",
    "ува": "увы",
    "уважаемые": "уважаемый",
    "устал": "устать",
    "усталь": "устать",
    "уфимка": "уфимец",
    "уходим": "уходить",
    "форчек": "форчекинг",
    "хайлайты": "хайлайт",
    "хозяйка": "хозяин",
    "хоккеистка": "хоккеист",
    "хорошенько": "хорошо",
    "хотелось": "хотеться",
    "цвета": "цвет",
    "целое": "целый",
    "частенько": "часто",
    "челленджа": "челлендж",
    "чемпионка": "чемпион",
    "чередует": "чередовать",
    "честной": "честный",
    "чувствовалось": "чувствоваться",
    "шажок": "шаг",
    "шайб": "шайба",
    "шарик": "шар",
    "шт": "штука",
    "date-го": "date",
    "org-арена": "арена",
}



"""
Функции предобработки текста.

Преобразование текста для подачи на вход нейронной модели.
"""

import json
from itertools import groupby
from pathlib import Path
from typing import Dict, List, Literal, Optional, Union

from natasha import Doc, NewsMorphTagger
from natasha.doc import DocToken

from khl.utils import emb, morph_vocab, segmenter
from khl.wrong_lemmas import fixed_lemmas

PLACEHOLDER = ""
UNKNOWN = "???"


morph_tagger = NewsMorphTagger(emb)


Word = str  # pragma: no mutate
Lemma = str  # pragma: no mutate
Code = int  # pragma: no mutate
Ner = Literal["per", "org", "loc", "date", "pen"]  # pragma: no mutate


def _merge(text_list: List[Word], source_word: Ner, target_word: Word) -> List[Word]:
    """
    'Схлопывает' одинаковые соседние заданные слова внутри списка в одно целевое.

    _merge(
        text_list=['1', '1', 'word', 'word', 'word', '1', 'word'],
        source_word='word',
        target_word='words'
    ) -> ['1', '1', 'words', '1', 'word']
    """
    merged_words_text_list = []
    for word, grouper in groupby(text_list):
        group = list(grouper)
        if word == source_word and len(group) > 1:
            merged_words_text_list.append(target_word)
        else:
            merged_words_text_list.extend(group)
    return merged_words_text_list


def _merge_pers(text_list: List[Word]) -> List[Word]:
    """['per', 'per', 'и', 'per', 'per'] -> ['pers', 'и', 'pers']."""
    return _merge(text_list=text_list, source_word="per", target_word="pers")


def _merge_orgs(text_list: List[Word]) -> List[Word]:
    """['org', 'org', 'и', 'org', 'org'] -> ['orgs', 'и', 'orgs']."""
    return _merge(text_list=text_list, source_word="org", target_word="orgs")


def _merge_locs(text_list: List[Word]) -> List[Word]:
    """['loc', 'loc', 'и', 'loc', 'loc'] -> ['locs', 'и', 'locs']."""
    return _merge(text_list=text_list, source_word="loc", target_word="locs")


def _merge_dates(text_list: List[Word]) -> List[Word]:
    """['date', 'date', 'и', 'date', 'date'] -> ['dates', 'и', 'dates']."""
    return _merge(text_list=text_list, source_word="date", target_word="dates")


def _merge_pens(text_list: List[Word]) -> List[Word]:
    """['pen', 'pen', 'и', 'pen', 'pen'] -> ['pens', 'и', 'pens']."""
    return _merge(text_list=text_list, source_word="pen", target_word="pens")


def _merge_ners(text_list: List[Word]) -> List[Word]:
    """# noqa
    ['per', 'per', 'org', 'org', 'loc', 'loc', 'date', 'date', 'pen', 'pen']
    -> ['pers', 'orgs', 'locs', 'dates', 'pens']
    """
    return _merge_pers(_merge_orgs(_merge_locs(_merge_dates(_merge_pens(text_list)))))


def _tokenize(text: str) -> List[DocToken]:
    """
    Разбивка текста на токены с морфемами.

    'с морфемами' означает, что у каждого токена определено
    к какой части речи токен принадлежит, в каком он роде, числе и падеже.
    Это нужно для дальнейшей лемматизации - приведению токена к начальной форме.
    """
    doc = Doc(text)
    doc.segment(segmenter)
    doc.tag_morph(morph_tagger)
    tokens: List[DocToken] = doc.tokens
    return tokens


def _merge_lemmas(lemmas: List[Lemma]) -> List[Lemma]:
    """
    Схлопывание одинаковых соседних лемм.

    ['и', 'и', 'или', 'или', 'или'] -> ['и', 'или'].
    """
    return [lemma for lemma, _ in groupby(lemmas)]


def fix_lemma(lemma: Lemma) -> Lemma:
    """
    Исправление неправильных лемм.

    'забиваем' -> 'забивать'
    'основный' -> 'основной'
    'родный'   -> 'родной'
    'голы'     -> 'гол'
    """
    return fixed_lemmas.get(lemma, lemma)


def lemmatize(text: str, stop_words_: Optional[List[Lemma]]) -> List[Lemma]:
    """
    Разбивка текста на леммы.

    Леммы - начальные формы слов (в нижнем регистре).
    Примеры:
      lemmatize(
        text="1 мая Морозов и Семин забили много голов от борта",
        stop_words_=None,
      ) -> ["1", "май", "морозов", "и", "семин", "забить", "много", "гол", "от", "борт"]
      lemmatize(
        text="1 мая Морозов и Семин забили много голов от борта",
        stop_words_=["и", "много", "от"],
      ) -> ["1", "май", "морозов", "семин", "забить", "гол", "борт"]
    """
    text_tokens = _tokenize(text)
    for token in text_tokens:
        token.lemmatize(morph_vocab)
    if stop_words_ is None:
        text_lemmas: List[Lemma] = [fix_lemma(token.lemma) for token in text_tokens]
    else:
        text_lemmas = [
            fixed_lemma
            for token in text_tokens
            if (fixed_lemma := fix_lemma(token.lemma)) not in stop_words_
        ]
    return _merge_lemmas(_merge_ners(text_lemmas))


def _merge_codes(codes: List[Code]) -> List[Code]:
    """
    Схлопывание одинаковых соседних кодов.

    [10, 10, 200, 200, 200] -> [10, 200].
    """
    return [code for code, _ in groupby(codes)]


def lemmas_to_codes(
    lemmas: List[Lemma],
    lemmas_coder: Dict[Lemma, Code],
    exclude_unknown: bool,
    max_len: Optional[int] = None,
) -> List[Code]:
    """
    Преобразует последовательность лемм в последовательность их кодов.

    exclude_unknown:
      если True, то леммы, которых нет в частотном словаре, отбрасываются;
      если False, то для лемм, которых нет в частотном словаре,
        проставляется код неизвестного слова
    """
    codes = []
    for lemma in lemmas:
        if not exclude_unknown:
            codes.append(lemmas_coder.get(lemma, lemmas_coder[UNKNOWN]))
        elif lemma in lemmas_coder:
            codes.append(lemmas_coder[lemma])
        else:
            continue
    codes = _merge_codes(codes)
    if max_len is None:
        return codes
    elif len(codes) >= max_len:  # pragma: no mutate
        return codes[:max_len]
    else:
        return _fill_placeholders(codes, lemmas_coder, max_len)


def _fill_placeholders(
    codes: List[Code],
    lemmas_coder: Dict[Lemma, Code],
    max_len: int,
) -> List[Code]:
    """Заполняет список кодов символами-заполнителями."""
    filled_codes = [lemmas_coder[PLACEHOLDER]] * (max_len - len(codes))
    filled_codes.extend(codes)
    return filled_codes


def codes_to_lemmas(codes: List[Code], lemmas_coder: Dict[Lemma, Code]) -> List[Lemma]:
    """Преобразует последовательность кодов в последовательность их лемм."""
    reversed_lemmas_coder = {value: key for key, value in lemmas_coder.items()}
    lemmas = []
    for code in codes:
        lemmas.append(reversed_lemmas_coder[code])
    return lemmas


def get_lemmas_coder(frequency_dictionary_file: Union[Path, str]) -> Dict[Lemma, Code]:
    """
    Получение словаря кодового представления лемм из частотного словаря лемм.

    frequency_dictionary_file - частотный словарь лемм:
      json-ка со словарем, где ключи - леммы,
      а значения - сколько раз данная лемма встретилась во всем датасете.
      Желательно, чтобы данный словарь был отсортирован в порядке убывания значений.
    Например:
      {".": 1000, "и": 500, "команда": 200, "гол": 100}

    Возвращает словарь (кодер), в котором каждой лемме присвоен свой уникальный код.
    Первые 2 элемента кодера зарезервированы:
      0 - символ-заполнитель
      1 - неизвестное слово
    """
    lemmas_coder = {PLACEHOLDER: 0, UNKNOWN: 1}
    with open(frequency_dictionary_file, "r", encoding="utf-8") as fr:
        freq_dict = json.load(fr)
    for freq, word in enumerate(freq_dict, len(lemmas_coder)):
        lemmas_coder[word] = freq
    return lemmas_coder



"""
Функции, определенным образом преобразующие хоккейные новости.

Нацелены на приведение текстов новостей из разных источников
к единому виду (унификацию), а также на избавление от элементов
текста новостей, не несущих особой смысловой нагрузки для
машинного обучения (упрощение).
"""


import re
from typing import List

from natasha import (
    DatesExtractor,
    Doc,
    MorphVocab,
    NewsEmbedding,
    NewsNERTagger,
    Segmenter,
)
from natasha.extractors import Match as NatashaMatch
from natasha.span import Span

from khl.teams_orgs import teams_orgs_pattern

segmenter = Segmenter()
morph_vocab = MorphVocab()  # pragma: no mutate
emb = NewsEmbedding()  # pragma: no mutate
ner_tagger = NewsNERTagger(emb)


def unify_text(text: str) -> str:
    """Приведение текстов новостей к единому виду."""
    for symbol in ['"', "`", "«", "»", "„", "“", "”"]:
        text = text.replace(symbol, "'")
    for symbol in ["—", "–", "−"]:
        text = text.replace(symbol, "-")
    text = (
        text.replace("⅛", "1/8")
        .replace("¼", "1/4")
        .replace("½", "1/2")
        .replace("й", "й")
        .replace("ё", "ё")
        .replace("…", "...")
    )
    text = re.sub(r"[^ А-Яа-яЁёA-Za-z0-9',.\[\]{}()/=+%№#@!?;:-]", " ", text)
    return merge_spaces(text).strip()


def _find_ners(text: str) -> List[Span]:
    """Нахождение именованных сущностей."""
    doc = Doc(text)
    doc.segment(segmenter)
    doc.tag_ner(ner_tagger)
    ners: List[Span] = doc.ner.spans
    return ners


def replace_ners(text: str) -> str:
    """
    Заменяет именованные сущности на их тип.

    'Иванов Иван'  -> 'per'
    'Магнитогорск' -> 'loc'
    'Ак Барс'      -> 'org'
    """
    ners_spans = _find_ners(text)
    for ner_span in reversed(ners_spans):
        text = text[: ner_span.start] + ner_span.type.lower() + text[ner_span.stop :]
    return text


def _find_dates(text: str) -> List[NatashaMatch]:
    dates_extractor = DatesExtractor(morph_vocab)
    return [match_ for match_ in dates_extractor(text)]


def replace_dates(text: str) -> str:
    """Заменяет дату на слово 'date'."""
    dates_matches = _find_dates(text)
    dates = [text[date_match.start : date_match.stop] for date_match in dates_matches]
    for date in dates:
        text = text.replace(date, "date", 1)
    return text


def _fix_space_plus_space(text: str) -> str:
    r"""
    Приведение к нормальному виду ' + '.

    '1 + 2'         -> '1+2'
    '1 +2+ 3'       -> '1+2+3'
    '1+ 2 +3'       -> '1+2+3'
    '1 + 2 + 3'     -> '1+2+3'
    '1  +  2  +  3' -> '1+2+3'
    '1\t+\t2'       -> '1+2'
    """
    pattern = r"\s*\+\s*"
    result = re.sub(pattern, "+", text)
    return result


def replace_penalty(text: str) -> str:
    """
    Заменяет штрафы формата 2+10 на слово 'pen'.

    '2+10', '2+20', '4+10', '4+20', '5+10', '5+20' -> 'pen'.
    """
    preprocessed_text = _fix_space_plus_space(text)
    pattern = r"\b(?<!\+)[245]\s*\+\s*(?:10|20)(?!\+)\b"
    result = re.sub(pattern, "pen", preprocessed_text)
    return result


def delete_numeric_data(text: str) -> str:
    """Удаление данных типа 12:25, 2-10."""
    pattern = r"\b\d+\s*(?:[:-]\s*\d+)+\b(?!-)"
    result = re.sub(pattern, "", text)
    return result


def _replace_dash_with_space(match_object: re.Match) -> str:  # type: ignore
    """Замена тире на пробел."""
    result: str = match_object.group(0).replace("-", " ")
    return result


def replace_dash_between_ners(text: str) -> str:
    """Замена тире между ner'ами на пробел."""
    pattern = r"(?:per|org|loc|date|pen)(?:\s*-+\s*(?:per|org|loc|date|pen))+"
    result = re.sub(pattern, _replace_dash_with_space, text)
    return result


def merge_spaces(text: str) -> str:
    """'Схлопывает' все соседние пробельные символы в один пробел."""
    return re.sub(r"\s{2,}", " ", text)


def leave_only_significant_symbols(text: str) -> str:
    """
    Оставление в тексте только имеющих вес символов.

    Производить непосредственно перед разбивкой текста на токены.

    Под "имеющие вес" подразумеваются символы, отсутствие или наличие которых,
    влияет на смысл текста, либо придает определенному участку
    текста какой-то дополнительный контекст. Например, двоеточиями
    выделяют диалоги, а знаки вопроса часто встречаются с упоминанием
    какого-то игрока в интервью со следующим контекстом - какого его
    самочувствие или выйдет ли он на игру в следующем матче.
    Пример:
      Иван Иванов? Он травмирован.
    """
    return re.sub(r"[^ А-Яа-яЁёA-Za-z:.?-]", " ", text)


def replace_exclamation_mark_with_dot(text: str) -> str:
    """
    Замена восклицательного знака в тексте на точку.

    '!?', '?!' оставляем без изменения, так как символ '!' потом
    отфильтруется функцией leave_only_significant_symbols.
    """
    text = re.sub(r"!+", "!", text)
    return re.sub(r"(?<!\?)!(?!\?)", ".", text)


def fix_dots(text: str) -> str:
    """
    Корректирование точек.

    '...' -> '.'
    ' .' -> '.'
    ' . . .' -> '.'
    ' - .' -> '.'
    """
    return re.sub(r"\s*\.*\-*(?:\s*\-*\s*\.)+", ".", text)


def fix_question_marks(text: str) -> str:
    """
    Корректирование вопросительных знаков.

    '???' -> '?'
    ' ?' -> '?'
    ' ? ? ?' -> '?'
    ' - ?' -> '?'
    """
    return re.sub(r"\s*\?*\-*(?:\s*\-*\s*\?)+", "?", text)


def generalize_top(text: str) -> str:
    """
    Обобщение ТОП'ов.

    'ТОП-3', 'ТОП-5', 'ТОП-10' -> 'ТОП'
    'TOP-3', 'TOP-5', 'TOP-10' -> 'ТОП'
    """
    pattern = r"\b(ТОП|топ|TOP|top)-?\d+\b"
    return re.sub(pattern, "ТОП", text)


def delete_serial_numbers(text: str) -> str:
    """Удаление порядковых числительных типа '5-й', '2ого', '1.'."""
    pattern = r"\b\d(?:\d*-?)*(?:[а-яёА-ЯЁё]{1,4}\b|\.)"
    return re.sub(pattern, "", text)


def delete_parentheses_content(text: str) -> str:
    """Удаление скобок вместе с их содержимым."""
    pattern = r"\([^()]+\)"
    return re.sub(pattern, "", text)


def delete_overtime_mark(text: str) -> str:
    """Удаление пометки овертайма."""
    pattern = r"(?<=\d)(?:ОТ|от|OT|ot\d?)|(?<=\s)\d?(?:ОТ|OT)\d?\b"
    return re.sub(pattern, "", text)


def delete_play_format(text: str) -> str:
    """
    Удаления формата игры.

    '5 на 5', '5 на 4', '5 на 3', '4 на 5', '3 на 5',
    '4 на 4', '4 на 3', '3 на 4', '3 на 3',
    '5х5', '5 х 4', '5Х3', '4 Х 5' и т.д.
    Функция должна вызываться после функции delete_numeric_data.
    """
    pattern = r"\b(?:3|4|5|6)\s*(?:на|НА|х|Х|x|X)\s*(?:3|4|5|6)\b"
    return re.sub(pattern, "", text)


def lowercase_shaiba_word(text: str) -> str:
    """
    'Шайба' -> 'шайба'.

    Natasha распознает слово 'Шайба' как ner.
    """
    return re.sub(r"\bШайба\b", "шайба", text)


def _latin_c_to_cirillic(match_object: re.Match) -> str:  # type: ignore
    """Замена латинского слова 'с' на кириллическое'с'."""
    if match_object.group(0) == "C":
        return "С"
    return "с"


def latin_c_to_cirillic(text: str) -> str:
    """Замена латинского слова 'с' на кириллическое'с'."""
    pattern = r"\b[cC]\b"
    return re.sub(pattern, _latin_c_to_cirillic, text)


def replace_vs_with_dash(text: str) -> str:
    """'vs' -> '-', '- vs - ' -> '-'."""
    pattern = r"\s*-?\s*(?<!\w)[Vv][sS](?!\w)\s*-?\s*"
    return re.sub(pattern, " - ", text)


def delete_urls(text: str) -> str:
    """
    Удаление всех ссылок из текста.

    Взято отсюда:
      https://stackoverflow.com/questions/839994/extracting-a-url-in-python
    """
    pattern = (
        r"\b(?:https?://)?(?:www\.)?(?:[\da-zа-яё\.-]+)\."
        r"(?:[a-zа-яё]{2,6})(?:/[\w\.-?=&]*)*/?\b"
    )
    return re.sub(pattern, "", text)


def handwritten_replace_orgs(text: str) -> str:
    """Костыльная замена названий чего-то в кавычках на org."""
    pattern = r"(?<=')[A-ZА-ЯЁ][a-zA-Zа-яА-ЯёЁ]+(?=')"
    return re.sub(pattern, "org", text)


def replace_concrete_orgs(text: str) -> str:
    """Замена прописанных названий лиг и команд на org."""
    return re.sub(teams_orgs_pattern, "org", text)


def fix_covid(text: str) -> str:
    """'COVID-19' -> 'covid'."""
    pattern = r"covid[+-]?\d*"
    return re.sub(pattern, "covid", text, flags=re.IGNORECASE)


def _latin_c_to_cirillic_in_word(match_object: re.Match) -> str:  # type: ignore
    """Замена латинской 'C' на кириллическую в слове."""
    word: str = match_object.group(0)
    return word.replace("C", "С").replace("c", "с")


def fix_latin_c_in_russian_words(text: str) -> str:
    """Замена латинской 'C' на кириллическую в русских словах."""
    pattern = r"\b(?:[А-Яа-яёЁ]*[cC][а-яА-ЯёЁcC]+|[А-Яа-яёЁ]+[cC][а-яА-ЯёЁcC]*)\b"
    return re.sub(pattern, _latin_c_to_cirillic_in_word, text)


def _cirillic_c_to_latin_in_word(match_object: re.Match) -> str:  # type: ignore
    """Замена кириллической 'С' на латинскую в слове."""
    word: str = match_object.group(0)
    return word.replace("С", "C").replace("с", "c")


def fix_cirillic_c_in_english_words(text: str) -> str:
    """Замена кириллической 'С' на латинскую в английских словах."""
    pattern = r"\b(?:[a-zA-Z]*[сС][a-zA-ZсС]+|[a-zA-Z]+[сС][a-zA-ZсС]*)\b"
    return re.sub(pattern, _cirillic_c_to_latin_in_word, text)


def _delete_lower_cirillic_letters_from_word(
    match_object: re.Match,  # type: ignore
) -> str:
    """
    Удаление подряд идущих строчных кириллических букв из слова.

    'COVIDом' -> 'COVID'
    'HIFKи'   -> 'HIFK'
    'PSка'    -> 'PS'
    'матч'    -> ''
    """
    word = match_object.group(0)
    return re.sub(r"[а-яё]+", "", word)


def delete_cirillic_ending_from_english_words(text: str) -> str:
    """
    Удаление строчных русских окончаний у заглавных английских слов.

    'COVIDом' -> 'COVID'
    'HIFKи'   -> 'HIFK'
    'PSка'    -> 'PS'
    """
    pattern = r"\b[A-Z]{2,}[а-яё]+\b"
    return re.sub(pattern, _delete_lower_cirillic_letters_from_word, text)


def _leave_only_cirillic(match_object: re.Match) -> str:  # type: ignore
    """Оставление только кириллических символов."""
    word = match_object.group(0)
    return re.sub(r"[^а-яА-ЯёЁ]", "", word)


def fix_english_dash_russian_words(text: str) -> str:
    """
    Оставление только русского слова из связки 'английское-русское'.

    'VIP-гость'      -> 'гость'
    'telegram-канал' -> 'канал'
    'YouTube-видео'  -> 'канал'
    """
    pattern = r"\b[a-zA-Z]+-[а-яА-ЯёЁ]+\b"
    return re.sub(pattern, _leave_only_cirillic, text)


def delete_age_category(text: str) -> str:
    """Удаление возрастной категории типа 'U-18'."""
    pattern = r"\s*-?U\s*-?\s*\d{1,2}"
    return re.sub(pattern, "", text)


def delete_birth_mark(text: str) -> str:
    """
    Удаление года рождения формата.

    'г.р.', '2000 г.р.', '2000/04 г. р.', '2000/2001 гг.р.'
    """
    pattern = r"\d+[/-]?\d*\s*гг?\.?\s*р\."
    return re.sub(pattern, "", text)


def delete_letter_dot_letter_dot(text: str) -> str:
    """Удаление надписей 'P.S.'."""
    pattern = r"\b[a-zA-Zа-яА-ЯёЁ]\.\s*[a-zA-Zа-яА-ЯёЁ]\."
    return re.sub(pattern, "", text)


def delete_shutouts(text: str) -> str:
    """
    Удаление надписей буллитов.

    Таких как 'SO', 'БУЛ', 'Б'.
    """
    pattern = (
        r"(?:\b[Ss][Oo]\b)|(?:\b[Бб][Уу][Лл](?:\.|\b))|"
        r"(?:\b[Бб](?:\.|\b))|(?<=\d)[Бб](?:\.|\b)"
    )
    return re.sub(pattern, "", text)


def delete_amplua(text: str) -> str:
    """
    Удаление сокращений амплуа.

    Таких как 'з.', 'н.', 'вр.'.
    """
    pattern = r"\b(?:[зн]|вр)(?:\s*\.|\b)"
    return re.sub(pattern, "", text)


def replace_tak_kak(text: str) -> str:
    """
    Замена сокращения 'т.к.'.

    'т.к.' -> 'так как'.
    """
    pattern = r"\bт\.?\s*\.?к(?:\s*\.|\b)"
    return re.sub(pattern, "так как", text)


def replace_to_est(text: str) -> str:
    """
    Замена сокращения 'т.е.'.

    'т.е.' -> 'то есть'.
    """
    pattern = r"\bт\.?\s*\.?е(?:\s*\.|\b)"
    return re.sub(pattern, "то есть", text)


def delete_quotes_with_one_symbol(text: str) -> str:
    """Удаление кавычек, содержащий в себе один символ, вместе с содержимым."""
    pattern = r"'\w'"
    return re.sub(pattern, "", text)


def delete_one_symbol_english_words(text: str) -> str:
    """Удаление слов из одной латинской буквы."""
    pattern = r"\b[a-zA-Z]\b"
    return re.sub(pattern, "", text)


def _delete_dash(match_object: re.Match) -> str:  # type: ignore
    """Удаление тире из строки."""
    result: str = match_object.group(0).replace("-", "")
    return result


def delete_beginning_ending_dashes_in_words(text: str) -> str:
    """Удаление тире в начале или в конце слова."""
    pattern = (
        r"(?<![a-zA-Zа-яА-ЯёЁ])-+[a-zA-Zа-яА-ЯёЁ]+-*(?![a-zA-Zа-яА-ЯёЁ])|"
        r"(?<![a-zA-Zа-яА-ЯёЁ])-*[a-zA-Zа-яА-ЯёЁ]+-+(?![a-zA-Zа-яА-ЯёЁ])"
    )
    return re.sub(pattern, _delete_dash, text)


def _split_ners(match_object: re.Match) -> str:  # type: ignore
    """
    Разделение слипшихся ner'ов.

    'perper' -> 'per per'
    'orgper' -> 'org per'
    и т.д.
    """
    string: str = match_object.group(0)
    if re.match(r"per|org|loc|pen", string):
        return string[:3] + " " + string[3:]
    else:
        return string[:4] + " " + string[4:]


def split_ners(text: str) -> str:
    """
    Разделение слипшихся ner'ов.

    'perper' -> 'per per'
    'orgper' -> 'org per'
    и т.д.
    """
    pattern = r"\b(?:per|org|loc|date|pen)(?:per|org|loc|date|pen)"
    return re.sub(pattern, _split_ners, text)


def _delete_spaces(match_object: re.Match) -> str:  # type: ignore
    """Удаление пробелов."""
    string: str = match_object.group(0)
    return string.replace(" ", "")


def fix_b_o_lshii(text: str) -> str:
    """
    В новостях КХЛ много слов 'б о льший'.

    Похоже буква 'о' с ударением так парсится с сайта.
    """
    pattern = r"\b[Бб] о льш"
    return re.sub(pattern, _delete_spaces, text)


def merge_dashes(text: str) -> str:
    """'Схлопывает' все соседние тире в один пробел."""
    return re.sub(r"(?<=\s)-+(\s*-+)+(?=\s)|-{2,}", "-", text)


def fix_question_dot(text: str) -> str:
    """'?..' -> '?'."""
    return re.sub(r"\?+[.\s]*\.", "?", text)


def fix_dot_question(text: str) -> str:
    """'..?' -> '?'."""
    return re.sub(r"[.\s]*\.\s*\?+", "?", text)


def delete_year_city_mark(text: str) -> str:
    """
    Удаление пометки года/города.

    '2021г.'        -> '2021'
    '2021/2022 гг.' -> '2021/2022 '
    '2018-2020 г.'  -> '2018-2020 '
    'г.Пенза'  -> 'Пенза'
    'г. Минск'  -> ' Минск'
    """
    return re.sub(r"(?<![а-яА-ЯёЁ])[Гг][Гг]?(?:\.|(?![а-яА-ЯёЁ]))", "", text)


def _surround_dash_with_spaces(match_object: re.Match) -> str:  # type: ignore
    """Расставление пробела по обе стороны от тире."""
    string: str = match_object.group(0)
    return merge_spaces(string.replace("-", " - "))


def fix_surname_dash_surname_dash_surname(text: str) -> str:
    """
    'Иванов-Петров-Сидоров' -> 'Иванов - Петров - Сидоров'.

    Natasha так распознает per'ов намного лучше.
    Работает для 3-х и более фамилий. Для 2-х уже будут баги,
    так как под данную регулярку попадают и другие сущности,
    например, команда 'Локо-Юниор' или город 'Нур-Султан'.
    """
    pattern = r"[A-ZА-ЯЁ][a-zа-яё]+(?:\s*-+\s*[A-ZА-ЯЁ][a-zа-яё]+){2,}"
    return re.sub(pattern, _surround_dash_with_spaces, text)


def fix_dash_word(text: str) -> str:
    """
    ' -Иванов' -> ' - Иванов', 'Иванов- ' -> 'Иванов -'.

    Natasha так распознает per'ов намного лучше.
    """
    pattern = r"\s*\B-+[A-ZА-ЯЁ]|[A-ZА-ЯЁ][a-zа-яё]+-+\B\s*"
    return re.sub(pattern, _surround_dash_with_spaces, text).strip()


def fix_org_loc(text: str) -> str:
    """
    'org loc' -> 'org'.

    Например, 'Динамо Рига' преобразуется в 'org loc',
    но нам в данном случае неважен loc, поэтому нужно
    оставлять просто org.
    """
    pattern = r"org[-\s]*loc"
    return re.sub(pattern, "org", text)


def delete_ending_colon_dash(text: str) -> str:
    """'Текст : - ' -> 'Текст'."""
    return text.rstrip(" -:")


def simplify_text(
    text: str,
    replace_ners_: bool,
    replace_dates_: bool,
    replace_penalties_: bool,
) -> str:
    """
    Упрощение текста хоккейной новости.

    Избавление от элементов текста новости, не несущих
    особой смысловой нагрузки для машинного обучения.

    Пример использования:
      simplify_text(
        text="1 января 2020 года Иван Иванов в Москве забил гол в ворота Спартака, а также заработал 5+10 за грубость",  # noqa
        replace_ners_=True,
        replace_dates_=True,
        replace_penalties=True,
      ) -> "date per в loc забил гол в ворота org а также заработал pen за грубость"

    Последовательность действий:
      1. Удаляем все что в скобках
      2. Заменяем сокращение 'т.к.'
      3. Заменяем сокращение 'т.е.'
      4. Удаляем слова формата 'И.о.', 'P.S.', 'т.д.'
      5. Исправляем слово 'б о льший'
      6. Удаляем надписи буллитов
      7. Удаляем ОТ
      8. Удаляем сокращения амплуа
      9. Заменяем слово 'Шайба' на 'шайба' (иначе Natasha распознает его как ner)
      10. Заменяем латинское слово 'c' на кириллическое 'с'
      11. Исправляем латинскую 'c' на кириллическую в русских словах
      12. Исправляем кириллическую 'с' на латинскую в английских словах
      13. Заменяем слово 'vs' на ' - '
      14. Удаляем строчные русские окончания у заглавных английских слов
      15. Заменяем COVID-19 на covid
      16. Оставляем только русское слово из связки 'английское-русское'
      17. Удаляем возрастные категории типа U-18
      18. Удаляем годы рождения
      19. Корректирум 'Иванов-Петров-Сидоров' -> 'Иванов - Петров - Сидоров'
      20. Корректируем ' -Иванов' -> ' - Иванов', 'Иванов- ' -> 'Иванов - '
      21. Заменяем ner'ы (если необходимо)
      22. Заменяем даты (если необходимо)
      23. Заменяем удаления (если необходимо)
      24. Удаляем пометки годов
      25. Костыльная замена названий чего-то в кавычках на org
      26. Замена прописанных названий лиг и команд на org
      27. Разделяем слипшиеся ner'ы
      28. Удаляем ссылки
      29. Удаляем кавычки с одним символом внутри
      30. Удаляем английские слова, состоящие только из одной буквы
      31. Обобщаем ТОП
      32. Удаляем числовые данные
      33. Удаляем порядковые числительные
      34. Удаляем формат игры ('5x5', '3 на 4' и т.д.)
      35. Заменяем восклицательные знаки на точки
      36. Оставляем только нужные символы
      37. 'org loc' -> 'org'
      38. Схлопываем пробелы
      39. Схлопываем тире
      40. Заменяем тире между ner'ами на пробел
      ('этот шаг в редких случаях приводит к неправильному схлопыванию ner'ов)
      41. Удаление тире в начале или в конце слова
      42. Корректируем точки
      43. Корректируем вопросительные знаки
      44. Корректируем '?..' -> '?'
      45. Корректируем '..?' -> '?'
      46. Удаляем тире и двоеточия в конце текста (rstrip)
    """
    text = delete_parentheses_content(text)
    text = replace_tak_kak(text)
    text = replace_to_est(text)
    text = delete_letter_dot_letter_dot(text)
    text = fix_b_o_lshii(text)
    text = delete_shutouts(text)
    text = delete_overtime_mark(text)
    text = delete_amplua(text)
    text = lowercase_shaiba_word(text)
    text = latin_c_to_cirillic(text)
    text = fix_latin_c_in_russian_words(text)
    text = fix_cirillic_c_in_english_words(text)
    text = replace_vs_with_dash(text)
    text = delete_cirillic_ending_from_english_words(text)
    text = fix_covid(text)
    text = fix_english_dash_russian_words(text)
    text = delete_age_category(text)
    text = delete_birth_mark(text)
    text = fix_surname_dash_surname_dash_surname(text)
    text = fix_dash_word(text)
    if replace_ners_:
        text = replace_ners(text)
    if replace_dates_:
        text = replace_dates(text)
    if replace_penalties_:
        text = replace_penalty(text)
    text = delete_year_city_mark(text)
    if replace_ners_:
        text = handwritten_replace_orgs(text)
        text = replace_concrete_orgs(text)
    text = split_ners(text)
    text = delete_urls(text)
    text = delete_quotes_with_one_symbol(text)
    text = delete_one_symbol_english_words(text)
    text = generalize_top(text)
    text = delete_numeric_data(text)
    text = delete_serial_numbers(text)
    text = delete_play_format(text)
    text = replace_exclamation_mark_with_dot(text)
    text = leave_only_significant_symbols(text)
    text = fix_org_loc(text)
    text = merge_spaces(text)
    text = merge_dashes(text)
    text = replace_dash_between_ners(text)
    text = delete_beginning_ending_dashes_in_words(text)
    text = fix_dots(text)
    text = fix_question_marks(text)
    text = fix_question_dot(text)
    text = fix_dot_question(text)
    text = delete_ending_colon_dash(text)
    return merge_spaces(text).strip()



"""Шаблон регулярных выражений для ручной замены названий команд и организаций."""


teams_orgs_pattern = r"""(?x)
\b(?:

# Названия лиг
(?i:
    # Континентальная хоккейная лига
        кхл|
        khl|
    # Высшая хоккейная лига
        вхл|
        vhl|
    # Молодежная хоккейная лига
        мхл|
        mhl|
    # Женская хоккейная лига
        жхл|
        zhl|
    # Национальная хоккейная лига
        нхл|
        nhl|
    # Американская хоккейная лига
        ахл|
        ahl
)|

# Названия команд КХЛ (вместе с "закрепившимися" за ними именами)
(?i:(?:hc|хк)\s*)?
(?:
    # Авангард
        А(?i:вангард)|
        A(?i:vangard)|
    # Автомобилист
        А(?i:втомобилист)|
        A(?i:vtomobilist)|
    # Адмирал
        А(?i:дмирал)|
        A(?i:dmiral)|
    # Ак Барс
        А[Кк]\s*(?i:барс)|
        A[Kk]\s*(?i:bars)|
    # Амур
        А(?i:мур)|
        A(?i:mur)|
    # Барыс
        Б(?i:арыс)|
        B(?i:arys)|
    # Витязь
        В(?i:итязь?)|
        V(?i:ityaz)|
    # Динамо (Рига, Минск, Москва)
        Д(?i:инамо)(?:\s*(?i:р|рига|м|мск|москва|мн|минск)\b)?|
        D(?i:[iy]namo)(?:\s*(?i:r|riga|m|msk|moscow|mn|minsk)\b)?|
    # Йокерит
        Й(?i:окерит)|
        J(?i:okerit)|
    # Куньлунь РС
        К(?i:унь?лунь?)(?:\s*(?i:рс|ред\s*стар)\b)?|
        K(?i:unlun)(?:\s*(?i:rs|red\s*star)\b)?|
    # Локомотив
        Л(?i:окомотив)|
        L(?i:okomotiv)|
    # Металлург
        М(?i:етал?лург)(?:\s*(?i:мг|магнитогорск)\b)?|
        (?i:магнитк[а-я]+)\b|
        (?i:ммг)\b|
        M(?i:etal?lurg)(?:\s*(?i:mg|magnitogorsk)\b)?|
        (?i:magnitk[a-z]+)\b|
        (?i:mmg)\b|
    # Нефтехимик
        Н(?i:ефтехимик)|
        N(?i:eftek?himik)|
    # Салават Юлаев
        С(?i:алават)(?i:[а-яё]*\s*юлаев)?|
        (?i:сю)\b|
        S(?i:alavat)(?i:(?:\'?[а-яёa-z])*\s*ulaev)?|
        (?i:su)\b|
    # Северсталь
        С(?i:еверсталь?)|
        S(?i:everstal)|
    # Сибирь
        С(?i:ибирь?)|
        S(?i:ibir)|
    # СКА
        (?i:ска)\b|
        (?i:ska)\b|
    # Спартак
        С(?i:партак)|
        S(?i:partak)|
    # Торпедо НН
        Т(?i:орпедо)(?:\s*(?i:нн|нижний\s*новгород)\b)?|
        T(?i:orpedo)(?:\s*(?i:nn|nizhni[yi]\s*novgorod)\b)?|
    # Трактор
        Т(?i:рактор)|
        T(?i:ra[kc]tor)|
    # ХК Сочи
        С(?i:очи)|
        S(?i:ochi)|
    # ЦСКА
        (?i:цска)\b|
        (?i:cska)\b|
    # team
        name
        # ПОСЛЕДНЕЕ НАЗВАНИЕ ПРОПИСАТЬ БЕЗ ВЕРТИКАЛЬНОЙ ЧЕРТОЧКИ!!!
)
)(?:\'?[а-яА-ЯёЁa-zA-Z])*
"""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['khl']

package_data = \
{'': ['*']}

install_requires = \
['natasha==1.4.0']

setup_kwargs = {
    'name': 'khl',
    'version': '1.0.1',
    'description': 'Preparing russian hockey news for machine learning',
    'long_description': '![Khl Logo](https://raw.githubusercontent.com/Rishat-F/khl/master/data/logo.png)\n\n<h1 align="center">No Water - Ice Only</h1>\n\nPreparing russian hockey news for machine learning.\n\n**Unify -> Simplify -> Preprocess** text and feed your neural model.\n\n## Installation\n\n*Khl* is available on PyPI:\n\n```console\n$ pip install khl\n```\nIt requires Python 3.8+ to run.\n\n## Usage\n\nTo get started right away with basic usage:\n\n```python\nfrom khl import text_to_codes\n\nlemmas_coder = {\n    \'\': 0,     # placeholder\n    \'???\': 1,  # unknown\n    \'.\': 2,\n    \'и\': 3,\n    \'в\': 4,\n    \'-\': 5,\n    \':\': 6,\n    \'матч\': 7,\n    \'за\': 8,\n    \'забить\': 9,\n    \'гол\': 10,\n    \'per\': 11,   # person entity\n    \'org\': 12,   # organization entity\n    \'loc\': 13,   # location entity\n    \'date\': 14,  # date entity\n    \'против\': 15,\n    \'год\': 16,\n    \'pers\': 17,  # few persons entity\n    \'orgs\': 18,  # few organizations entity\n    \'свой\': 19\n}\n\ntext = """\n    1 апреля 2023 года в Москве в матче ⅛ финала против „Спартака” Иван Иванов забил свой 100—й гол за карьеру.\n    «Динамо Мск» - «Спартак» 2:1 ОТ (1:0 0:1 0:0 1:0) Голы забили: Иванов, Петров и Сидоров.\n"""\n\ncodes = text_to_codes(\n    text=text,\n    lemmas_coder=lemmas_coder,\n    stop_words_=["в", "за", "и", "свой"],  # stop words to drop\n    replace_ners_=True,                    # replace named entities ("Иван Иванов" -> "per", "Спартак" -> "org", "Москва" -> "loc")\n    replace_dates_=True,                   # replace dates ("1 апреля 2023 года" -> "date")\n    replace_penalties_=True,               # replace penalties ("5+20" -> "pen")\n    exclude_unknown=True,                  # drop lemma that not presented in lemmas_coder\n    max_len=20,                            # get sequence of codes of length 20\n)\n# codes = [0, 0, 0, 0, 0, 14, 13, 7, 15, 12, 11, 9, 10, 2, 18, 10, 9, 6, 17, 2]\n```\n\n```text_to_codes``` is a very high level function. What\'s happens under hood see in [Lower level usage](#lower-level-usage).\n\n## What is `lemmas_coder`?\n`lemmas_coder` is just a dictionary where each lemma is represented with unique integer code.\nNote that first two elements are reserved for *placeholder* and *unknown* elements.\n\nIt is possible to get `lemmas_coder` from frequency dictionary file (see in [Get lemmas coder](#2-get-lemmas-coder)).\nFrequency dictionary file is a **json**-file with dictionary where key is lemma and value is how many times this lemma occurred in your whole dataset.\nPreferably it should be sorted in descending order of values.  \n`example_frequency_dictionary.json`:\n\n```json\n{\n  ".": 1000,\n  "и": 500,\n  "в": 400,\n  "-": 300,\n  ":": 300,\n  "матч": 290,\n  "за": 250,\n  "забить": 240,\n  "гол": 230,\n  "per": 200,\n  "org": 150,\n  "loc": 150,\n  "date": 100,\n  "против": 90,\n  "год": 70,\n  "pers": 40,\n  "orgs": 30,\n  "свой": 20\n}\n```\n\nYou could make and use your own frequency dictionary or download [this dictionary](https://github.com/Rishat-F/khl/blob/master/data/frequency_dictionary.json) created by myself.\n\n## Lower level usage<a id="lower-level-usage"></a>\n\n#### 1. Make imports\n```python\nfrom khl import stop_words\nfrom khl import utils\nfrom khl import preprocess\n```\n\n#### 2. Get lemmas coder<a id="2-get-lemmas-coder"></a>\n```python\nlemmas_coder = preprocess.get_lemmas_coder("example_frequency_dictionary.json")\n```\n\n#### 3. Define text\n```python\ntext = """\n    1 апреля 2023 года в Москве в матче ⅛ финала против „Спартака” Иван Иванов забил свой 100—й гол за карьеру.\n    «Динамо Мск» - «Спартак» 2:1 ОТ (1:0 0:1 0:0 1:0) Голы забили: Иванов, Петров и Сидоров.\n"""\n```\n\n#### 4. Unify\n```python\nunified_text = utils.unify_text(text)\n# "1 апреля 2023 года в Москве в матче 1/8 финала против \'Спартака\' Иван Иванов забил свой 100-й гол за карьеру. \'Динамо Мск\' - \'Спартак\' 2:1 ОТ (1:0 0:1 0:0 1:0) Голы забили: Иванов, Петров и Сидоров."\n```\n\n#### 5. Simplify\n```python\nsimplified_text = utils.simplify_text(\n    text=unified_text,\n    replace_ners_=True,\n    replace_dates_=True,\n    replace_penalties_=True,\n)\n# \'date в loc в матче финала против org per забил свой гол за карьеру. org org Голы забили: per per и per.\'\n```\n\n#### 6. Lemmatize\n```python\nlemmas = preprocess.lemmatize(text=simplified_text, stop_words_=stop_words)\n# [\'date\', \'loc\', \'матч\', \'финал\', \'против\', \'org\', \'per\', \'забить\', \'гол\', \'карьера\', \'.\', \'orgs\', \'гол\', \'забить\', \':\', \'pers\', \'.\']\n```\n\n#### 7. Transform to codes\n```python\ncodes = preprocess.lemmas_to_codes(\n    lemmas=lemmas,\n    lemmas_coder=lemmas_coder,\n    exclude_unknown=True,\n    max_len=20,\n)\n# [0, 0, 0, 0, 0, 14, 13, 7, 15, 12, 11, 9, 10, 2, 18, 10, 9, 6, 17, 2]\n```\n\n#### 8. Transform to lemmas back (just to look which lemmas are presented in codes sequence)\n```python\nprint(\n    preprocess.codes_to_lemmas(codes=codes, lemmas_coder=lemmas_coder)\n)\n# [\'\', \'\', \'\', \'\', \'\', \'date\', \'loc\', \'матч\', \'против\', \'org\', \'per\', \'забить\', \'гол\', \'.\', \'orgs\', \'гол\', \'забить\', \':\', \'pers\', \'.\']\n```\n',
    'author': 'Rishat Fayzullin',
    'author_email': 'nilluziaf@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Rishat-F/khl',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
